{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load firstboot_extras %}
{% load static %}


{# Adapt mainmenu-links during firstboot #}
{% block mainmenu_left %}

  {% firstboot_is_completed as firstboot_completed %}
  {% if not firstboot_completed %}

    <span class="navbar-brand">
      <img src="{% static 'theme/img/freedombox-logo-32px.png' %}"
           alt="{{ cfg.box_name }}" />
      <a href="{% url 'index' %}">
        {% blocktrans trimmed %}{{ box_name }} Setup{% endblocktrans %}
      </a>
    </span>

  {% else %}
    {{ block.super }}
  {% endif %}

{% endblock %}


{% block mainmenu_right %}

  {% firstboot_is_completed as firstboot_completed %}
  {% if not firstboot_completed %}
    {% include "firstboot_navbar.html" %}
  {% else %}
    {{ block.super }}
  {% endif %}

{% endblock %}
