{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

<ul class="nav nav-tabs">
  {% for urlitem in subsubmenu %}
    <li class="nav-item" role="presentation">
      <a class="{% if urlitem.active %} active {% endif %} nav-link"
         href="{{ urlitem.url }}">{{ urlitem.text }}</a>
    </li>
  {% endfor %}
</ul>
