{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block content %}

  <h3>{{ title }}</h3>

  <form class="form form-connection-edit" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    <input type="submit" class="btn btn-primary"
           value="{% trans "Edit Connection" %}"/>
  </form>

{% endblock %}

{% block page_js %}

  <script type="text/javascript" src="{% static 'networks/networks.js' %}"></script>

{% endblock %}
