{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <h3>{{ title }}</h3>

  <form class="form" method="post">
    {% csrf_token %}

    {% if form.ssh_public_key|length == 0 %}
      <p>
        {% blocktrans trimmed %}
          Could not reach SSH host {{ hostname }}. Please verify that the host
          is up and accepting connections.
        {% endblocktrans %}
      </p>
      {% if form.keyscan_error %}
        <pre class="alert alert-danger">{{ form.keyscan_error }}</pre>
      {% endif %}
    {% else %}
      <p>
        {% blocktrans trimmed %}
          The authenticity of SSH host {{ hostname }} could not be established.
          The host advertises the following SSH public keys. Please verify any
          one of them.
        {% endblocktrans %}
      </p>

      <section>
        <p>
          <a class="btn btn-default collapsed collapsible-button"
             data-toggle="collapse" href="#help" aria-expanded="false">
            <span class="fa fa-chevron-right fa-fw" aria-hidden="true"></span>
            {% trans "How to verify?" %}
          </a>
        </p>
        <div class="collapse card-body" id="help">
          <p>
            {% blocktrans trimmed %}
              Run the following command on the SSH host machine. The output
              should match one of the provided options. You can also use DSA,
              ECDSA, Ed25519 etc. instead of RSA, by choosing the corresponding
              file.
            {% endblocktrans %}
          </p>
          <p>
            <code>ssh-keygen -lf /etc/ssh/ssh_host_rsa_key.pub</code>
          </p>
        </div>
      </section>

      {{ form|bootstrap }}

      <input type="submit" class="btn btn-primary" value="{% trans 'Verify Host' %}"/>
    {% endif %}
  </form>

{% endblock %}
