{% extends "base_firstboot.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block content %}
  <h2>{% trans "Software Update" %}</h2>

  {% if is_busy %}
    <div class="upgrades-status-frame clearfix">
      <div class="upgrade-status-icon pull-left">
        <span class="fa fa-refresh fa-spin fa-3x fa-pull-left processing"></span>
      </div>
      <p>
        <strong>{% trans "Updating, please wait..." %}</strong>
      </p>
      <p>
	{% blocktrans trimmed %}
	  <strong>This may take a long time to complete.</strong> During
	  an update, this web interface may be temporarily unavailable and
	  show an error. In that case, refresh the page to continue.
	{% endblocktrans %}
      </p>
    </div>
  {% else %}
    <p>
      {% blocktrans %}
	{{ box_name }} is up to date. Press Next to continue.
      {% endblocktrans %}
    </p>
    <a title="{% trans 'Next' %}"
       role="button" class="btn btn-primary"
       href={% url next_step %}>
      {% trans 'Next' %}
    </a>
  {% endif %}

{% endblock %}
