{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block internal_zone_warning %}
  {% for component in firewall %}
    {% if not component.is_external %}
      <div class="alert alert-warning" role="alert">
        {% blocktrans trimmed with service_name=component.name %}
          <em>{{ service_name }}</em> is available only on internal networks or when the client is connected to {{ box_name }} through VPN.
        {% endblocktrans %}
        <p>
          {% with interfaces=component.get_internal_interfaces %}
            {% if not interfaces %}
              {% trans "Currently there are no network interfaces configured as internal." %}
            {% else %}
              {% blocktrans trimmed with interface_list=interfaces|join:", " %}
                Currently the following network interfaces are configured as internal: {{ interface_list }}
              {% endblocktrans %}
            {% endif %}
          {% endwith %}
        </p>
      </div>
    {% endif %}
  {% endfor %}
{% endblock %}
