{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% for operation in operations %}
  <div class="app-operation">
    <span class="fa fa-refresh fa-spin processing"></span>
    {{ operation.translated_message }}
  </div>
{% endfor %}
