{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <h2>{% trans "Setup Complete!" %}</h2>

  <p>
    {% blocktrans trimmed %}
      Without any apps, your {{ box_name }} cannot do very much.
    {% endblocktrans %}
  </p>

  <div class="text-center">
    <a class="btn btn-lg btn-primary" href="{% url 'apps' %}">
      {% trans "Install Apps" %}</a>
  </div>

  <p>
    <p>
      {% url 'networks:index' as networks_url %}
      {% blocktrans trimmed %}
        You may want to check the <a href="{{ networks_url }}">network
        setup</a> and modify it if necessary.
      {% endblocktrans %}
    </p>
  </p>

{% endblock %}
