{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <h3>{{ title }}</h3>

  <form class="form" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    <div class="alert alert-warning" role="alert">
      {% blocktrans trimmed %}
      The credentials for this repository are stored on your {{ box_name }}.
      <br />
      To restore a backup on a new {{ box_name }} you need the SSH
      credentials and, if chosen, the encryption passphrase.
      {% endblocktrans %}
    </div>

    <input type="submit" class="btn btn-primary"
           value="{% trans "Create Location" %}"/>
  </form>

{% endblock %}
