{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block configuration %}
  <div id="snapshot-not-supported" class="alert alert-danger">
    {% blocktrans trimmed with fs_types_supported|join:', ' as types_supported %}
      You have a filesystem of type <strong>{{ fs_type }}</strong>. Snapshots
      are currently only available on <strong>{{ types_supported }}</strong>
      filesystems.
    {% endblocktrans %}
  </div>
{% endblock %}
