{% extends 'app.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load static %}

{% block extra_content %}

  <h3>{% trans "Diagnostics Run" %}</h3>
  <div class="btn-toolbar">
    <form class="form form-diagnostics-full" method="post"
          action="{% url 'diagnostics:full' %}">
      {% csrf_token %}
      <input type="submit" class="btn btn-default"
             value="{% trans "Run Diagnostics Now" %}"/>
    </form>

    {% if results_available %}
      <a class="btn btn-default" role="button"href="{% url 'diagnostics:full' %}">
        {% trans "View Results" %}
      </a>
    {% endif %}
  </div>

{% endblock %}
